/**
 * Created by eeee on 17/10/8.
 */


/**
 * 创建成语面板
 * @param txtArr            //文字数组
 * @param wordNum           //可输入文字数量
 * @param confirmCallback   //点击确定回调(返回一个参数【当前输入的文本】)
 * @returns {*}
 */
function createIdiomView(txtArr,wordNum,confirmCallback){

    if(!txtArr)return;

    var wordArr = [],   //当前输入的文字
        labelArr = [];

    var view = createLevelSprite("t2_com_box1.png");
    view.callback=confirmCallback;

    //失败回调
    view.gameOver=function(){
        showWrong(this.x-55,this.y+200);
    }

    //成功回调
    view.gameWin=function(){
        showRight(this.x-55,this.y+200)
    }

    txtArr=arrUpSet(txtArr);


    for (var i = 0; i < txtArr.length; i++) {
        var txt = txtArr[i];

        var colorSprite = createRectSprite(50,50);
        colorSprite.attr({
            anchorX: 0,
            x:i%8*70+17,
            y:Math.floor(i/8)*85+53,
            opacity:0
        });
        view.addChild(colorSprite);
        colorSprite.txt = txt;
        addTouchCallBack(colorSprite, function () {
            if(wordArr.length>=wordNum)return;

            wordArr.push(this.txt);

            removeListener(this);

            labelArr[wordArr.length-1].string = wordArr[wordArr.length-1] || "";
        });
        var label1 = createLabel(txt, 40);
        label1.attr({
            x: colorSprite.width / 2,
            y: colorSprite.height / 2
        });
        colorSprite.addChild(label1);


        //创建答案输入框
        if(i<wordNum){
            var box=createLevelSprite("t2_com_box2.png");
            box.attr({
                x: view.width/2 - 28 + i * (92+60-wordNum*12) - 110*wordNum/2,
                y: view.height+60
            });
            view.addChild(box);

            if(wordNum>=5){
                box.x+=40;
            }

            var label2 = createLabel("", 40);
            label2.attr({
                x: box.width/2,
                y: box.height/2
            });
            box.addChild(label2);
            labelArr.push(label2);
        }
    }


    var button = createButton({img:"t2_com_box3.png",txt:"确定",txtSize:30,txtColor:cc.color(62, 65, 67)});
    button.attr({
        x:view.width/2+220,
        y:view.height+60
    });
    button.callback = function () {
        if(typeof(view.callback)=="function")
            view.callback(wordArr.join(""));
    };
    view.addChild(button);


    if(cc.winSize.width/cc.winSize.height<0.53){
        view.scale=1-2*Math.ceil(53-(cc.winSize.width/cc.winSize.height)*100)/100;
    }

    return view;
}

/**
 *创建数字选择面板
 * @param confirmCallback   //点击确定后回调
 */
function numSelectView(confirmCallback){
    var curNum=0;

    var view=createLevelSprite("t2_com_box3.png");
    view.callback=confirmCallback;

    //失败回调
    view.gameOver=function(){
        showWrong(this.x,this.y);
    }

    //成功回调
    view.gameWin=function(){
        showRight(this.x+20,this.y);
    }

    var numLabel=createLabel(""+curNum);
    numLabel.attr({
        x:view.width/2,
        y:view.height/2
    });
    view.addChild(numLabel);

    //减号按钮
    var subtractBtn=createButton({img:"t2_com_jian.png"});
    subtractBtn.attr({
        x:view.width/2-115,
        y:view.height/2
    });
    view.addChild(subtractBtn);
    subtractBtn.callback=function(){
        if(curNum<=0)return;
        curNum--;
        numLabel.string=""+curNum;
    }


    //加号按钮
    var addBtn=createButton({img:"t2_com_jia.png"});
    addBtn.attr({
        x:view.width/2+115,
        y:view.height/2
    });
    view.addChild(addBtn);
    addBtn.callback=function(){
        if(curNum>=999)return;
        curNum++;
        numLabel.string=""+curNum;
    }

    //确定按钮
    var comfirmBtn=createButton({img:"t2_com_box3.png",txt:"确定",txtSize:30,txtColor:cc.color(62, 65, 67)});
    comfirmBtn.attr({
        x:view.width/2,
        y:-40
    });
    view.addChild(comfirmBtn);
    comfirmBtn.callback=function(){
        if(typeof(view.callback)=="function")
            view.callback(curNum);
    }

    return view;
}

/**
 * 创建选择按钮
 * @param param
 *
 * 参数示例1：（文字按钮）
 * param={
 *     A:"答案1",
 *     B:"答案2",
 *     C:"答案3",
 *     D:"答案4",
 *     answer:"B"
 * }
 *
 * 参数示例2：（图片按钮）
 * param={
 *     A:{img:"photo1.png"},
 *     B:{img:"photo2.png"},
 *     C:{img:"photo3.png"},
 *     D:{img:"photo4.png"},
 *     answer:"B"
 * }
 *
 * 参数示例3：（图片+文字）
 * param={
 *     A:{img:"photo1.png",txt:"答案1"},
 *     B:{img:"photo2.png",txt:"答案2"},
 *     C:{img:"photo3.png",txt:"答案3"},
 *     D:{img:"photo4.png",txt:"答案4"},
 *     answer:"B"
 * }
 *
 * 参数示例4：（修改失败文案，输入失败文案的键值）
 * param={
 *     A:{img:"photo1.png",txt:"答案1",overKey:"lossWord1"},
 *     B:{img:"photo2.png",txt:"答案2",overKey:"lossWord2"},
 *     C:{img:"photo3.png",txt:"答案3",overKey:"lossWord3"},
 *     D:{img:"photo4.png",txt:"答案4",overKey:"lossWord4"},
 *     answer:"B"
 * }
 *
 */
function createAnswerSelectView(param){
    var defaultParam={
        A:"答案1",
        B:"答案2",
        C:"答案3",
        D:"答案4",
        answer:"B", //答案
        callback:function(isBingo){},   //点击按钮后回调
        autoTxtSize:true,   //按钮里面文字是否自动缩放
        isShowFrame:true,   //是否显示底框【仅对文本按钮有效，设置后变成竖排】
        isShowNum:true,     //是否显示编号
        modeType:0  //排版模式(1：竖排，2：竖排大框，其他：默认)【仅对文本按钮有效】
    };

    param = param || defaultParam;

    var view=new cc.Node();

    var paramType=0;    //参数类型，1：文字按钮，2：图片按钮，3：图片+文字按钮

    switch(typeof(param["A"])){
        case "string":
            paramType=1;
            break;
        case "object":
            if(param["A"]["img"] && param["A"]["txt"] ){
                paramType=3;
            }else if(param["A"]["img"]){
                paramType=2;
            }else if(param["A"]["txt"]){
                paramType=1;
            }
            break;
    }

    cc.log("当前选择按钮类型：",paramType);

    var answerList=["A","B","C","D"];
    var btnList=[];
    var imgMaxSize={width:0,height:0};

    var autoTxtSize=true;
    if(param["autoTxtSize"]===false)
        autoTxtSize=false;

    var txtColor=cc.color(62, 65, 67);

    switch(paramType){
        case 1:
            var btnList=[];
            var isTouch=true;
            var labelBtnMaxWidth=0;
            for(var i=0;i<answerList.length;i++){
                var curAnswer=answerList[i];

                var answerTxt=param[curAnswer];
                if(typeof(answerTxt)=="object")
                    answerTxt=param[curAnswer]["txt"];

                if(param["isShowFrame"]===false){
                    var labelBtn=createLabel(answerTxt, 36, cc.color(255, 255, 255));
                    labelBtn.answer = curAnswer;
                    view.addChild(labelBtn);
                    btnList.push(labelBtn);
                    addTouchCallBack(labelBtn,function(){
                        if(!isTouch)return;
                        isTouch=false;

                        var convertPos = this.parent.convertToWorldSpace(cc.p(this.x, this.y));

                        if(typeof(param.callback)=="function")
                            param.callback(this.answer==param.answer);

                        if (this.answer == param.answer) {
                            showRight(convertPos.x, convertPos.y);
                        } else {
                            showWrong(convertPos.x, convertPos.y);
                        }
                    },true,function(touch){
                        var convertPos=this.convertToNodeSpace(touch.getLocation());
                        var rect=cc.rect(0,0,this.width,this.height);
                        if(cc.rectContainsPoint(rect,convertPos)){
                            this.scale=1.05;
                        }else{
                            this.scale=1;
                        }
                    });
                    labelBtn.beganCallback=function(){
                        this.scale=1.05;
                    }
                    labelBtn.endCallback=function(){
                        this.scale=1;
                    }

                    labelBtnMaxWidth=Math.max(labelBtnMaxWidth,labelBtn.width);
                }else {
                    var boxImg = "t2_com_kuang.png";

                    if (param["modeType"] == 2)
                        boxImg = "t2_com_box5.png";


                    var txtBtn = createButton({
                        img: boxImg,
                        txt: answerTxt,
                        txtSize: 36,
                        txtColor: txtColor,
                        autoTxtSize: autoTxtSize
                    });
                    view.addChild(txtBtn);
                    txtBtn.answer = curAnswer;
                    txtBtn.callback = function () {
                        var convertPos = this.parent.convertToWorldSpace(cc.p(this.x, this.y));

                        if(typeof(param.callback)=="function")
                            param.callback(this.answer==param.answer);

                        if (this.answer == param.answer) {
                            showRight(convertPos.x, convertPos.y);
                        } else {
                            showWrong(convertPos.x, convertPos.y);
                        }
                    };
                    btnList.push(txtBtn);
                }
            }

            //按钮定位,添加序号
            for(var i=0;i<btnList.length;i++){
                var btn=btnList[i];
                if(param["isShowFrame"]===false) {
                    btn.attr({
                        x: -(labelBtnMaxWidth-btn.width)/2,
                        y: i * -(btn.height + 30)
                    });
                }else{
                    switch (param["modeType"]) {
                        case 1:
                            btn.attr({
                                x: 0,
                                y: i * -(btn.height + 20)
                            });
                            break;
                        case 2:
                            btn.attr({
                                x: 0,
                                y: i * -(btn.height + 30)
                            });
                            break;
                        default:
                            btn.attr({
                                x: -160 + i % 2 * 320,
                                y: Math.floor(i / 2) * -120
                            });
                            break;
                    }
                }

                if(param["isShowNum"]!==false) {
                    btn.x += 30;
                    var number = createLabel(btn.answer + "、", 36, cc.color(255, 255, 255));
                    number.attr({
                        anchorX: 1,
                        x: btn.x - btn.width / 2,
                        y: btn.y
                    });
                    view.addChild(number);
                }
            }
            break;
        case 2:
            for(var i=0;i<answerList.length;i++){
                var curAnswer=answerList[i];

                var imgBtn=createButton({img:param[curAnswer]["img"]});
                view.addChild(imgBtn);
                imgBtn.answer=curAnswer;
                imgBtn.callback=function(){
                    var convertPos=this.parent.convertToWorldSpace(cc.p(this.x,this.y));

                    if(typeof(param.callback)=="function")
                        param.callback(this.answer==param.answer);

                    if(this.answer==param.answer){
                        showRight(convertPos.x,convertPos.y);
                    }else{
                        showWrong(convertPos.x,convertPos.y,null,param[curAnswer]["overKey"]);
                    }
                }
                btnList.push(imgBtn);

                imgMaxSize.width=Math.max(imgBtn.width,imgMaxSize.width);
                imgMaxSize.height=Math.max(imgBtn.height,imgMaxSize.height);

            }

            for(var j=0;j<btnList.length;j++){
                btnList[j].attr({
                    x:-Math.max(imgMaxSize.width/2+30,135) + j%2 * Math.max(imgMaxSize.width/2+40,135)*2,
                    y:Math.floor(j/2)*-Math.max(imgMaxSize.height+40,200)
                });

                if(param["isShowNum"]!==false) {
                    btnList[j].x += 30;
                    var number = createLabel(answerList[j] + "、", 36, cc.color(255, 255, 255));
                    number.attr({
                        anchorX: 1,
                        anchorY: 0,
                        x: btnList[j].x - imgMaxSize.width / 2 + 15,
                        y: btnList[j].y - imgMaxSize.height / 3
                    });
                    view.addChild(number);
                }
            }
            break;
        case 3:
            for(var i=0;i<answerList.length;i++){
                var curAnswer=answerList[i];

                var imgBtn=createButton({img:param[curAnswer]["img"]});
                view.addChild(imgBtn);
                imgBtn.answer=curAnswer;
                imgBtn.callback=function(){
                    var convertPos=this.parent.convertToWorldSpace(cc.p(this.x,this.y));
                    if(this.answer==param.answer){
                        showRight(convertPos.x,convertPos.y);
                    }else{
                        showWrong(convertPos.x,convertPos.y,null,param[curAnswer]["overKey"]);
                    }
                }

                var txtBtn=createButton({
                    img:"t2_com_kuang.png",
                    txt:param[curAnswer]["txt"],
                    txtSize:36,
                    txtColor:txtColor,
                    autoTxtSize:autoTxtSize
                });
                view.addChild(txtBtn);
                txtBtn.answer=curAnswer;
                txtBtn.callback=function(){
                    var convertPos=this.parent.convertToWorldSpace(cc.p(this.x,this.y));

                    if(typeof(param.callback)=="function")
                        param.callback(this.answer==param.answer);

                    if(this.answer==param.answer){
                        showRight(convertPos.x,convertPos.y);
                    }else{
                        showWrong(convertPos.x,convertPos.y,null,param[curAnswer]["overKey"]);
                    }
                }

                btnList.push({imgBtn:imgBtn,txtBtn:txtBtn});

                imgMaxSize.width=Math.max(imgBtn.width,imgMaxSize.width);
                imgMaxSize.height=Math.max(imgBtn.height,imgMaxSize.height);

            }

            for(var j=0;j<btnList.length;j++){
                btnList[j]["imgBtn"].attr({
                    anchorY:0,
                    x:-Math.max(imgMaxSize.width/2+30,135) + j%2 * Math.max(imgMaxSize.width/2+50,135)*2,
                    y:Math.floor(j/2)*-Math.max(imgMaxSize.height+100,200)
                });

                btnList[j]["txtBtn"].attr({
                    anchorY:1,
                    x:btnList[j]["imgBtn"].x,
                    y:btnList[j]["imgBtn"].y-10
                });

                if(param["isShowNum"]!==false) {
                    btnList[j].x += 30;
                    var number = createLabel(answerList[j] + "、", 36, cc.color(255, 255, 255));
                    number.attr({
                        x: btnList[j]["txtBtn"].x - btnList[j]["txtBtn"].width / 2 - 20,
                        y: btnList[j]["txtBtn"].y - btnList[j]["txtBtn"].height / 2
                    });
                    view.addChild(number);
                }
            }
            break;
    }

    if(cc.winSize.width/cc.winSize.height<0.53){
        view.scale=1-2*Math.ceil(53-(cc.winSize.width/cc.winSize.height)*100)/100;
    }

    return view;
}

/**
 * 添加圈圈
 * @param target
 * @param x
 * @param y
 * @param scale
 * @param scaleY
 */
function addCircles(target,x,y,scale,scaleY){
    var circles=createLevelSprite("t2_com_quan.png");
    circles.attr({
        x:x || winCenter().x,
        y:y || winCenter().y,
        scaleX:scale || 1,
        scaleY:scaleY || scale || 1
    });
    target.addChild(circles,99);

    return circles;
}